/*
 * Xfer Auto Path
 * Copyright (C) 2008 Konrad Gräfe
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301, USA.
 */

#include "config.h"

#define PLUGIN_PREFS_PREFIX "/plugins/gtk/xfer_auto_path"

#ifndef PURPLE_PLUGINS
#define PURPLE_PLUGINS
#endif

#include "internal.h"

#include <unistd.h>

#include <plugin.h>
#include <version.h>

#include <debug.h>

#include <request.h>
#include <ft.h>
#include <gtkft.h>

#include <gtkblist.h>

static PurplePlugin *plugin;

static gboolean ensure_path_exists(const char *dir) {
        if (!g_file_test(dir, G_FILE_TEST_IS_DIR))
        {
                if (purple_build_dir(dir, S_IRUSR | S_IWUSR | S_IXUSR))
                        return FALSE;
        }

        return TRUE;
}


static gchar *strreplace_stripped(gchar* string, const gchar *delimiter, const gchar *replacement) {
	gchar *ret;
	gchar *replacement_stripped, *tmp;

	tmp = g_strdup(replacement);
	replacement_stripped = g_strstrip(tmp);

	ret = purple_strreplace(string, delimiter, replacement_stripped);

	g_free(tmp);

	return ret;
}

static void file_recv_request_cb(PurpleXfer *xfer, gpointer handle) {
	gchar *path, *tmp, *filename;
	const gchar *account_username, *account_alias;
	const gchar *buddy_username, *buddy_alias;
	const char *contact_alias;
	const gchar *protocol;

	PurpleBuddy *buddy;
	PurpleContact *contact;
	PurpleBlistNode *parent;

	if(!xfer) return;
	if(!xfer->account) return;

	account_username = xfer->account->username;
	account_alias = xfer->account->alias;
	if(!account_alias) account_alias = account_username;

	buddy = purple_find_buddy(xfer->account, xfer->who);
	if(buddy) {
		buddy_username = buddy->name;
		buddy_alias = buddy->alias;
		if(!buddy_alias) buddy_alias = buddy_username;
	
		parent=((PurpleBlistNode *)buddy)->parent;
		if(parent && PURPLE_BLIST_NODE_IS_CONTACT(parent)) {
			contact=(PurpleContact *)parent;
			contact_alias=contact->alias;
			if(!contact_alias) contact_alias=purple_contact_get_priority_buddy(contact)->alias;
			if(!contact_alias) contact_alias=purple_contact_get_priority_buddy(contact)->name;
		} else {
			contact_alias=buddy_alias;
		}
	} else {
		return;
	}
	
	protocol = purple_account_get_protocol_name(xfer->account);

	
	path = g_strdup(purple_prefs_get_string(PLUGIN_PREFS_PREFIX "/directory"));

	tmp=path;
	path = strreplace_stripped(tmp, "%b", buddy_username);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%B", buddy_alias);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%C", contact_alias);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%a", account_username);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%A", account_alias);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%p", protocol);
	g_free(tmp);

	tmp=path;
	path = strreplace_stripped(tmp, "%u", purple_user_dir());
	g_free(tmp);

	if(ensure_path_exists(path)) {
		purple_prefs_set_string("/pidgin/filelocations/last_save_folder", path);

		if(purple_prefs_get_bool(PLUGIN_PREFS_PREFIX "/auto_accept")) {
			xfer->status=PURPLE_XFER_STATUS_ACCEPTED;
		
			filename=g_build_filename(path, xfer->filename, NULL);
			if(!g_file_test(filename, G_FILE_TEST_EXISTS)) {
				purple_xfer_request_accepted(xfer, filename);
			}
			g_free(filename);
		}
	}

	g_free(path);
}

static void drawing_tooltip_cb(PurpleBlistNode *node, GString *str, gboolean full, void *data) {
	const gchar *account_username, *account_alias;
	const gchar *buddy_username, *buddy_alias;
	const char *contact_alias;
	const gchar *protocol;

	PurpleBuddy *buddy;
	PurpleContact *contact;
	PurpleBlistNode *parent;
	
	//if(!PURPLE_BLIST_NODE_IS_CONTACT(node) &&
	//   !PURPLE_BLIST_NODE_IS_BUDDY(node)) return;
	if(!PURPLE_BLIST_NODE_IS_BUDDY(node)) return;
	
	buddy = (PurpleBuddy *) node;

	account_username = buddy->account->username;
	account_alias = buddy->account->alias;
	if(!account_alias) account_alias = account_username;

	
	buddy_username = buddy->name;
	buddy_alias = buddy->alias;
	if(!buddy_alias) buddy_alias = buddy_username;

	parent=((PurpleBlistNode *)buddy)->parent;
	if(parent && PURPLE_BLIST_NODE_IS_CONTACT(parent)) {
		contact=(PurpleContact *)parent;
		contact_alias=contact->alias;
		if(!contact_alias) contact_alias=purple_contact_get_priority_buddy(contact)->alias;
		if(!contact_alias) contact_alias=purple_contact_get_priority_buddy(contact)->name;
	} else {
		contact_alias=buddy_alias;
	}
	
	protocol = purple_account_get_protocol_name(buddy->account);

	purple_debug_info(PLUGIN_STATIC_NAME, "buddy_username: %s\n", buddy_username);
	purple_debug_info(PLUGIN_STATIC_NAME, "buddy_alias: %s\n", buddy_alias);
	purple_debug_info(PLUGIN_STATIC_NAME, "contact_alias: %s\n", contact_alias);
	purple_debug_info(PLUGIN_STATIC_NAME, "account_username: %s\n", account_username);
	purple_debug_info(PLUGIN_STATIC_NAME, "account_alias: %s\n", account_alias);
	purple_debug_info(PLUGIN_STATIC_NAME, "protocol: %s\n", protocol);
	purple_debug_info(PLUGIN_STATIC_NAME, "purple_user_dir(): %s\n", purple_user_dir());
}

static gboolean plugin_load(PurplePlugin *_plugin) {
	plugin=_plugin;
	
	purple_signal_connect(pidgin_blist_get_handle(), "drawing-tooltip", plugin, PURPLE_CALLBACK(drawing_tooltip_cb), NULL);
	purple_signal_connect(purple_xfers_get_handle(), "file-recv-request", plugin, PURPLE_CALLBACK(file_recv_request_cb), plugin);

	
	return TRUE;
}

static gboolean plugin_unload(PurplePlugin *plugin) {
	return TRUE;
}

static PurplePluginPrefFrame * get_plugin_pref_frame(PurplePlugin *plugin) {
	PurplePluginPrefFrame *frame;
	PurplePluginPref *ppref;
	
	frame = purple_plugin_pref_frame_new();

	ppref = purple_plugin_pref_new_with_name_and_label(PLUGIN_PREFS_PREFIX "/directory", _("Directory:"));
        purple_plugin_pref_set_type(ppref, PURPLE_PLUGIN_PREF_NONE);
        purple_plugin_pref_frame_add(frame, ppref);

	ppref = purple_plugin_pref_new_with_name_and_label(PLUGIN_PREFS_PREFIX "/auto_accept", _("Auto accept if file does not exist"));
	purple_plugin_pref_frame_add(frame, ppref);

	ppref = purple_plugin_pref_new_with_label(_("Available placeholders:"));
	purple_plugin_pref_frame_add(frame, ppref);

	ppref = purple_plugin_pref_new_with_label(_("%p\t- Protocol\n%a\t- Account Username\n%A\t- Account Alias\n%b\t- Buddy Username\n%B\t- Buddy Alias\n%C\t- Contact Alias\n%u\t- User Directory"));
        purple_plugin_pref_set_type(ppref, PURPLE_PLUGIN_PREF_INFO);
        purple_plugin_pref_frame_add(frame, ppref);

	return frame;
}

static PurplePluginUiInfo prefs_info = {
        get_plugin_pref_frame,
        0,   /* page_num (Reserved) */
        NULL, /* frame (Reserved) */
        /* Padding */
        NULL,
        NULL,
        NULL,
        NULL
};

static PurplePluginInfo info = {
	PURPLE_PLUGIN_MAGIC,
	PURPLE_MAJOR_VERSION,
	PURPLE_MINOR_VERSION,
	PURPLE_PLUGIN_STANDARD,								/**< type           */
	NULL,												/**< ui_requirement */
	0,													/**< flags          */
	NULL,												/**< dependencies   */
	PURPLE_PRIORITY_DEFAULT,							/**< priority       */

	PLUGIN_ID,				/**< id             */
	NULL,												/**< name           */
	PLUGIN_VERSION,										/**< version        */
	NULL,												/**  summary        */
				
	NULL,												/**  description    */
	PLUGIN_AUTHOR,			/**< author         */
	PLUGIN_WEBSITE,
														/**< homepage       */

	plugin_load,										/**< load           */
	plugin_unload,										/**< unload         */
	NULL,												/**< destroy        */

	NULL,												/**< ui_info        */
	NULL,												/**< extra_info     */
	&prefs_info,										/**< prefs_info     */
	NULL,												/**< actions        */
	/* padding */
	NULL,
	NULL,
	NULL,
	NULL
};

static void init_plugin(PurplePlugin *plugin) {
	gchar *directory;

#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
#endif /* ENABLE_NLS */

	info.name        = _("Xfer Auto Path");
	info.summary     = _("Automatically set the folder for saving your incoming file transfers.");
	info.description     = _("This Plugin allows you to automatically set the folder for saving your incoming file transfers by some criteria such as buddy alias.");
	
	directory = g_build_filename("%u", "incoming", "%C", NULL);
	
	purple_prefs_add_none(PLUGIN_PREFS_PREFIX);
	purple_prefs_add_string(PLUGIN_PREFS_PREFIX "/directory", directory);
	purple_prefs_add_bool(PLUGIN_PREFS_PREFIX "/auto_accept", FALSE);
	
	g_free(directory);

}

PURPLE_INIT_PLUGIN(PLUGIN_STATIC_NAME, init_plugin, info)
